#include "CFThrottle.H"
#include "Screen.H"

#include "Logger.H"
extern Logger logger;
extern Screen *scr;

#define MAX_INPUTWAIT_MSECS         100000
#define MIN_INPUTWAIT_MSECS         0    
#define MAX_INPUTHANDLERWAIT_MSECS  100000
#define MIN_INPUTHANDLERWAIT_MSECS  0
#define MAX_REFRESHWAIT_MSECS       500000
#define MIN_REFRESHWAIT_MSECS       0

CFThrottle::CFThrottle(int wdth, int hght,bool brdr, bool shdw):
		Window(wdth,hght,brdr,shdw) {
	unsigned long
		rwait=0, iwait=0, ihwait=0; 

	_rebld=true;
	_wtitle="CPU Throttle";
	_inputLabel.style(_stile);
	_inputHandlerLabel.style(_stile);
	_refreshLabel.style(_stile);
	_label1.style(_stile);
	_label2.style(_stile);
	_label3.style(_stile);
	_inputLabel.setText("Input Poll Wait (microseconds)");
	_inputHandlerLabel.setText("Input Handler Wait (microseconds)");
	_refreshLabel.setText("Refresh Wait (microseconds)");
	_label1.setText("less CPU");
	_label2.setText("more CPU");
	_label3.setText("(hit <ENTER> to reset values)");
	_inputS=new Slider(wdth-6,1,-(MAX_INPUTWAIT_MSECS),MIN_INPUTWAIT_MSECS);
	_inputHandlerS=new Slider(wdth-6,1,-(MAX_INPUTHANDLERWAIT_MSECS),
			MIN_INPUTHANDLERWAIT_MSECS);
	_refreshS=new Slider(wdth-6,1,-(MAX_REFRESHWAIT_MSECS),
			MIN_REFRESHWAIT_MSECS);
	Window::add("inputlabel",&_inputLabel,3,2);
	Window::add("inputHandlerLabel",&_inputHandlerLabel,3,5);
	Window::add("refreshLabel",&_refreshLabel,3,8);
	Window::add("inputS",_inputS,3,3);
	Window::add("inputHandlerS",_inputHandlerS,3,6);
	Window::add("refreshS",_refreshS,3,9);
	Window::add("CPUlabel1",&_label1,3,11);
	Window::add("CPUlabel2",&_label2,(_width-3-_label2.length()),11);
	Window::add("CPUlabel3",&_label3,(_width-_label3.length())/2,12);
	_active=_inputS;
	(*scr).getTiming(rwait,iwait,ihwait);
	(*_inputS).setPos(-iwait);
	(*_inputHandlerS).setPos(-ihwait);
	(*_refreshS).setPos(-rwait);
}

CFThrottle::~CFThrottle() {
	delete (_inputS);
	delete (_inputHandlerS);
	delete (_refreshS);
}

widstat CFThrottle::input(Char inp) {
	unsigned long
		rwait=0, iwait=0, ihwait=0; 
	widstat
		rv;

	_rebld=true;
	if (inp.getSymbol() == 0x09) {
		if (_active==_inputS) {
			(*_active).outFocus();
			_active=_inputHandlerS;
			(*_active).inFocus();
		}
		else if (_active==_inputHandlerS) {
			(*_active).outFocus();
			_active=_refreshS;
			(*_active).inFocus();
		}
		else  {
			(*_active).outFocus();
			_active=_inputS;
			(*_active).inFocus();
		}
		return (ws_accepted);
	}
	if (inp.isEnter()) {
		(*scr).resetTiming();
		(*scr).getTiming(rwait,iwait,ihwait);
		(*_refreshS).setPos(-rwait);
		(*_inputS).setPos(-iwait);
		(*_inputHandlerS).setPos(-ihwait);
		return (ws_accepted);
	}
	rv=(*_active).input(inp);
	rwait=-(*_refreshS).getPos();
	iwait=-(*_inputS).getPos();
	ihwait=-(*_inputHandlerS).getPos();
	(*scr).adjustTiming(rwait,iwait,ihwait);
	return (rv);
}

widstat CFThrottle::style(Char stile, Char tstile) {
	_stile=stile;
	_inputLabel.style(_stile);
	_inputHandlerLabel.style(_stile);
	_refreshLabel.style(_stile);
	_label1.style(_stile);
	_label2.style(_stile);
	(*_inputS).style(_stile, tstile);
	(*_inputHandlerS).style(_stile, tstile);
	(*_refreshS).style(_stile, tstile);
	tstile.swap();
	_label3.style(tstile);
	_rebld=true;
	return (ws_accepted);
}

int CFThrottle::setValues(long rwait, long iwait, long ihwait) {
	if ((rwait != -1) &&
		(rwait >= MIN_REFRESHWAIT_MSECS) && (rwait <= MAX_REFRESHWAIT_MSECS))
		(*_refreshS).setPos(-rwait);
	if ((iwait != -1) &&
		(iwait >= MIN_INPUTWAIT_MSECS) && (iwait <= MAX_INPUTWAIT_MSECS))
		(*_inputS).setPos(-iwait);
	if ((ihwait != -1) &&
		(ihwait >= MIN_INPUTHANDLERWAIT_MSECS) &&
		(ihwait <= MAX_INPUTHANDLERWAIT_MSECS))
		(*_inputHandlerS).setPos(-ihwait);
	rwait=-(*_refreshS).getPos();
	iwait=-(*_inputS).getPos();
	ihwait=-(*_inputHandlerS).getPos();
	(*scr).adjustTiming(rwait,iwait,ihwait);
	return 0;
}

int CFThrottle::getValues(long &rwait, long &iwait, long &ihwait) {
	unsigned long urwait, uiwait, uihwait;
	(*scr).getTiming(urwait,uiwait,uihwait);
	rwait=urwait;
	iwait=uiwait;
	ihwait=uihwait;
	return 0;
}
